@@ -26,6 +26,7 @@
 #include "config.h"
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
@@ -67,27 +68,14 @@ static const gchar introspection_xml[] =
 G_DEFINE_TYPE(ObexAgent, obex_agent, G_TYPE_OBJECT)
 
 static ObexAgent *agent;
-static GDBusProxy *manager;
 static BluetoothClient *client;
 
 static AcceptSetting accept_setting = -1;
 static gboolean show_notifications = FALSE;
 
-static GtkStatusIcon *statusicon = NULL;
-static guint num_notifications = 0;
-
-static void
-hide_statusicon (void)
-{
-	num_notifications--;
-	if (num_notifications == 0)
-		gtk_status_icon_set_visible (statusicon, FALSE);
-}
-
 static void
 on_close_notification (NotifyNotification *notification)
 {
-	hide_statusicon ();
 	g_object_unref (notification);
 }
 
@@ -96,47 +84,52 @@ notification_launch_action_on_file_cb (NotifyNotification *notification,
 				       const char *action,
 				       const char *file_uri)
 {
-	GdkDisplay *display;
-	GAppLaunchContext *ctx;
-	GTimeVal val;
-
 	g_assert (action != NULL);
 
-	g_get_current_time (&val);
-
-	display = gdk_display_get_default ();
-	ctx = G_APP_LAUNCH_CONTEXT (gdk_display_get_app_launch_context (display));
-	gdk_app_launch_context_set_timestamp (GDK_APP_LAUNCH_CONTEXT (ctx), val.tv_sec);
-
 	/* We launch the file viewer for the file */
 	if (g_str_equal (action, "display") != FALSE) {
+		GdkDisplay *display;
+		GAppLaunchContext *ctx;
+		GTimeVal val;
+
+		g_get_current_time (&val);
+
+		display = gdk_display_get_default ();
+		ctx = G_APP_LAUNCH_CONTEXT (gdk_display_get_app_launch_context (display));
+		gdk_app_launch_context_set_timestamp (GDK_APP_LAUNCH_CONTEXT (ctx), val.tv_sec);
+
 		if (g_app_info_launch_default_for_uri (file_uri, ctx, NULL) == FALSE) {
 			g_warning ("Failed to launch the file viewer\n");
 		}
+
+		g_object_unref (ctx);
 	}
 
 	/* we open the Downloads folder */
 	if (g_str_equal (action, "reveal") != FALSE) {
-		GFile *file;
-		GFile *parent;
-		gchar *parent_uri;
+		GDBusConnection *connection = agent->connection;
+		GVariantBuilder builder;
 
-		file = g_file_new_for_uri (file_uri);
-		parent = g_file_get_parent (file);
-		parent_uri = g_file_get_uri (parent);
-		g_object_unref (file);
-		g_object_unref (parent);
+		g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
+		g_variant_builder_add (&builder, "s", file_uri);
 
-		if (!g_app_info_launch_default_for_uri (parent_uri, ctx, NULL)) {
-			g_warning ("Failed to launch the file manager\n");
-		}
+		g_dbus_connection_call (connection,
+					"org.freedesktop.FileManager1",
+					"/org/freedesktop/FileManager1",
+					"org.freedesktop.FileManager1",
+					"ShowItems",
+					g_variant_new ("(ass)", &builder, ""),
+					NULL,
+					G_DBUS_CALL_FLAGS_NONE,
+					-1,
+					NULL,
+					NULL,
+					NULL);
 
-		g_free (parent_uri);
+		g_variant_builder_clear (&builder);
 	}
 
 	notify_notification_close (notification, NULL);
-	/* No need to call hide_statusicon(), closing the notification
-	 * will call the close callback */
 }
 
 static void
@@ -158,11 +151,11 @@ show_notification (const char *filename)
 	notification_text = g_strdup_printf(_("You received \"%s\" via Bluetooth"), display);
 	g_free (display);
 	notification = notify_notification_new (_("You received a file"),
-								 notification_text,
-								 "dialog-information");
+						notification_text,
+						"bluetooth");
 
 	notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
-	notify_notification_set_hint_string (notification, "desktop-entry", "gnome-user-share-properties");
+	notify_notification_set_hint_string (notification, "desktop-entry", "gnome-bluetooth-panel");
 
 	mime_type = g_content_type_guess (filename, NULL, 0, NULL);
 	app = g_app_info_get_default_for_type (mime_type, FALSE);
@@ -194,14 +187,15 @@ show_notification (const char *filename)
 }
 
 static void
-show_icon (void)
+reject_transfer (GDBusMethodInvocation *invocation)
 {
-	if (statusicon == NULL) {
-		statusicon = gtk_status_icon_new_from_icon_name ("gnome-obex-server");
-	} else {
-		gtk_status_icon_set_visible (statusicon, TRUE);
-	}
-	num_notifications++;
+	const char *filename;
+
+	filename = g_object_get_data (G_OBJECT (invocation), "temp-filename");
+	g_remove (filename);
+
+	g_dbus_method_invocation_return_dbus_error (invocation,
+		"org.bluez.obex.Error.Rejected", "Not Authorized");
 }
 
 static void
@@ -215,8 +209,6 @@ ask_user_transfer_accepted (NotifyNotification *notification,
 
 	g_dbus_method_invocation_return_value (invocation,
 		g_variant_new ("(s)", file));
-
-	show_icon ();
 }
 
 static void
@@ -225,9 +217,7 @@ ask_user_transfer_rejected (NotifyNotification *notification,
 			    GDBusMethodInvocation *invocation)
 {
 	g_debug ("Notification: transfer rejected! rejecting transfer");
-
-	g_dbus_method_invocation_return_dbus_error (invocation,
-		"org.bluez.obex.Error.Rejected", "Not Authorized");
+	reject_transfer (invocation);
 }
 
 static void
@@ -235,9 +225,7 @@ ask_user_on_close (NotifyNotification *notification,
 		   GDBusMethodInvocation *invocation)
 {
 	g_debug ("Notification closed! rejecting transfer");
-
-	g_dbus_method_invocation_return_dbus_error (invocation,
-		"org.bluez.obex.Error.Rejected", "Not Authorized");
+	reject_transfer (invocation);
 }
 
 static void
@@ -255,7 +243,7 @@ ask_user (GDBusMethodInvocation *invocation,
 
 	notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
 	notify_notification_set_hint_string (notification, "desktop-entry",
-					     "gnome-user-share-properties");
+					     "gnome-bluetooth-panel");
 
 	notify_notification_add_action (notification, "receive", _("Receive"),
 					(NotifyActionCallback) ask_user_transfer_accepted,
@@ -387,8 +375,7 @@ on_session_acquired (GObject *object,
 
 out:
 	g_debug ("Rejecting transfer");
-	g_dbus_method_invocation_return_dbus_error (invocation,
-		"org.bluez.obex.Error.Rejected", "Not Authorized");
+	reject_transfer (invocation);
 }
 
 static void
@@ -415,8 +402,7 @@ check_if_bonded (GDBusProxy *transfer,
 	} else {
 		g_debug ("Could not get session path for the transfer, "
 			 "rejecting the transfer");
-		g_dbus_method_invocation_return_dbus_error (invocation,
-			"org.bluez.obex.Error.Rejected", "Not Authorized");
+		reject_transfer (invocation);
 	}
 }
 
@@ -481,80 +467,81 @@ parse_extension (const char *filename)
 }
 
 static char *
-get_destination_path (const char *filename)
-{
-  char *dest_dir;
-  char *destination_filename;
-
-  dest_dir = lookup_download_dir ();
-  destination_filename = g_build_filename (dest_dir, filename, NULL);
-  g_free (dest_dir);
-
-  /* Append (n) as needed. */
-  if (g_file_test (destination_filename, G_FILE_TEST_EXISTS)) {
-    int i = 1;
-    const char *dot_pos;
-    gssize position;
-    char *serial = NULL;
-    GString *tmp_filename;
-
-    dot_pos = parse_extension (destination_filename);
-    if (dot_pos)
-      position = dot_pos - destination_filename;
-    else
-      position = strlen (destination_filename);
-
-    tmp_filename = g_string_new (NULL);
-
-    do {
-      serial = g_strdup_printf ("(%d)", i++);
-
-      g_string_assign (tmp_filename, destination_filename);
-      g_string_insert (tmp_filename, position, serial);
-
-      g_free (serial);
-    } while (g_file_test (tmp_filename->str, G_FILE_TEST_EXISTS));
-
-    destination_filename = g_strdup (tmp_filename->str);
-    g_string_free (tmp_filename, TRUE);
-  }
-
-  return destination_filename;
-}
-
-static void
 move_temp_filename (GObject *object)
 {
 	const char *orig_filename;
-	char *dest_filename;
+	char *dest_filename, *dest_dir;
 	GFile *src, *dest;
 	GError *error = NULL;
 	gboolean res;
 
 	orig_filename = g_object_get_data (object, "temp-filename");
-	/* FIXME move to the destination directly, rather than
-	 * getting the name separately */
-	dest_filename = get_destination_path (g_object_get_data (object, "filename"));
-
 	src = g_file_new_for_path (orig_filename);
+
+	dest_dir = lookup_download_dir ();
+	dest_filename = g_build_filename (dest_dir, g_object_get_data (object, "filename"), NULL);
+	g_free (dest_dir);
 	dest = g_file_new_for_path (dest_filename);
 
-	/* This is sync, but the files will be on the same partition already
-	 * (~/.cache/obexd to ~/Downloads) */
 	res = g_file_move (src, dest,
 			   G_FILE_COPY_NONE, NULL,
 			   NULL, NULL, &error);
-	g_debug ("Moving %s (orig name %s) to %s",
-		 orig_filename, (char *) g_object_get_data (object, "filename"), dest_filename);
 
-	if (res == FALSE) {
+	/* This is sync, but the files will be on the same partition already
+	 * (~/.cache/obexd to ~/Downloads) */
+	if (!res && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+		guint i = 1;
+		const char *dot_pos;
+		gssize position;
+		char *serial = NULL;
+		GString *tmp_filename;
+
+		dot_pos = parse_extension (dest_filename);
+		if (dot_pos)
+			position = dot_pos - dest_filename;
+		else
+			position = strlen (dest_filename);
+
+		tmp_filename = g_string_new (NULL);
+		g_string_assign (tmp_filename, dest_filename);
+
+		while (!res && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+			g_debug ("Couldn't move file to %s", tmp_filename->str);
+
+			g_clear_error (&error);
+			g_object_unref (dest);
+
+			serial = g_strdup_printf ("(%d)", i++);
+
+			g_string_assign (tmp_filename, dest_filename);
+			g_string_insert (tmp_filename, position, serial);
+
+			g_free (serial);
+
+			dest = g_file_new_for_path (tmp_filename->str);
+			res = g_file_move (src, dest,
+					   G_FILE_COPY_NONE, NULL,
+					   NULL, NULL, &error);
+		}
+
+		g_free (dest_filename);
+		dest_filename = g_strdup (tmp_filename->str);
+		g_string_free (tmp_filename, TRUE);
+	}
+
+	if (!res) {
 		g_warning ("Failed to move %s to %s: '%s'",
 			   orig_filename, dest_filename, error->message);
 		g_error_free (error);
+	} else {
+		g_debug ("Moved %s (orig name %s) to %s",
+			 orig_filename, (char *) g_object_get_data (object, "filename"), dest_filename);
 	}
+
 	g_object_unref (src);
 	g_object_unref (dest);
-	g_free (dest_filename);
+
+	return dest_filename;
 }
 
 static void
@@ -584,13 +571,14 @@ transfer_property_changed (GDBusProxy *transfer,
 			g_debug ("Got status %s = %s for filename %s", status, str, filename);
 
 			if (g_str_equal (status, "complete")) {
-				move_temp_filename (G_OBJECT (transfer));
+				char *path;
+
+				path = move_temp_filename (G_OBJECT (transfer));
 				if (show_notifications) {
 					g_debug ("transfer completed, showing a notification");
-					show_notification (filename);
-				} else {
-					hide_statusicon ();
+					show_notification (path);
 				}
+				g_free (path);
 			}
 
 			/* Done with this transfer */
@@ -617,7 +605,6 @@ obex_agent_authorize_push (GObject *source_object,
 	const gchar *filename = g_variant_get_string (variant, NULL);
 	char *template;
 	int fd;
-	gboolean authorize = FALSE;
 
 	g_debug ("AuthorizePush received");
 
@@ -635,7 +622,7 @@ obex_agent_authorize_push (GObject *source_object,
 
 	switch (accept_setting) {
 	case ACCEPT_ALWAYS:
-		authorize = TRUE;
+		/* authorized! */
 		break;
 	case ACCEPT_BONDED:
 		check_if_bonded (transfer, invocation);
@@ -650,23 +637,14 @@ obex_agent_authorize_push (GObject *source_object,
 		/* ask_user() will accept or reject the transfer */
 		return;
 	default:
-		g_warn_if_reached ();
+		g_assert_not_reached ();
 	}
 
-	if (authorize) {
-		g_dbus_method_invocation_return_value (invocation,
-			g_variant_new ("(s)", template));
-
-		show_icon ();
-
-		g_debug ("Incoming transfer authorized: %s (temp file: %s)", filename, template);
-		g_free (template);
-	} else {
-		g_dbus_method_invocation_return_dbus_error (invocation,
-			"org.bluez.obex.Error.Rejected", "Not Authorized");
-		g_debug ("Incoming transfer rejected: %s", filename);
-	}
+	g_dbus_method_invocation_return_value (invocation,
+					       g_variant_new ("(s)", template));
 
+	g_debug ("Incoming transfer authorized: %s (temp file: %s)", filename, template);
+	g_free (template);
 	g_variant_unref (variant);
 }
 
@@ -713,6 +691,29 @@ static const GDBusInterfaceVTable interface_vtable =
   NULL
 };
 
+static void
+obexd_appeared_cb (GDBusConnection *connection,
+		   const gchar *name,
+		   const gchar *name_owner,
+		   gpointer user_data)
+{
+	ObexAgent *self = user_data;
+
+	g_debug ("obexd appeared, registering agent");
+	g_dbus_connection_call (self->connection,
+				MANAGER_SERVICE,
+				MANAGER_PATH,
+				MANAGER_IFACE,
+				"RegisterAgent",
+				g_variant_new ("(o)", AGENT_PATH),
+				NULL,
+				G_DBUS_CALL_FLAGS_NONE,
+				-1,
+				NULL,
+				NULL,
+				NULL);
+}
+
 static void
 on_bus_acquired (GDBusConnection *connection,
 		 const gchar     *name,
@@ -736,69 +737,29 @@ on_bus_acquired (GDBusConnection *connection,
 	g_dbus_node_info_unref (introspection_data);
 
 	g_assert (self->object_reg_id > 0);
-}
 
-static void
-on_agent_registered (GObject *source_object,
-		     GAsyncResult *res,
-		     gpointer user_data)
-{
-	GError *error = NULL;
-	GVariant *v;
-
-	v = g_dbus_proxy_call_finish (manager, res, &error);
-
-	if (error) {
-		g_warning ("error: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	g_variant_unref (v);
-}
-
-static void
-on_proxy_acquired (GObject *source_object,
-		   GAsyncResult *res,
-		   gpointer user_data)
-{
-	GError *error = NULL;
-
-	manager = g_dbus_proxy_new_for_bus_finish (res, &error);
-
-	if (!manager) {
-		g_warning ("error: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	g_dbus_proxy_call (manager,
-			   "RegisterAgent",
-			   g_variant_new ("(o)", AGENT_PATH),
-			   G_DBUS_CALL_FLAGS_NONE,
-			   -1,
-			   NULL,
-			   on_agent_registered,
-			   NULL);
-}
-
-static void
-on_name_acquired (GDBusConnection *connection,
-		  const gchar     *name,
-		  gpointer         user_data)
-{
-}
-
-static void
-on_name_lost (GDBusConnection *connection,
-	      const gchar     *name,
-	      gpointer         user_data)
-{
+	self->obexd_watch_id = g_bus_watch_name_on_connection (self->connection,
+							       MANAGER_SERVICE,
+							       G_BUS_NAME_WATCHER_FLAGS_NONE,
+							       obexd_appeared_cb,
+							       NULL,
+							       self,
+							       NULL);
 }
 
 static void
 obex_agent_init (ObexAgent *self)
 {
+	self->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+					 AGENT_IFACE,
+					 G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT,
+					 on_bus_acquired,
+					 NULL,
+					 NULL,
+					 self,
+					 NULL);
+
+	client = bluetooth_client_new ();
 }
 
 static void
@@ -807,7 +768,13 @@ obex_agent_dispose (GObject *obj)
 	ObexAgent *self = OBEX_AGENT (obj);
 
 	g_dbus_connection_unregister_object (self->connection, self->object_reg_id);
+	self->object_reg_id = 0;
+
 	g_bus_unown_name (self->owner_id);
+	self->owner_id = 0;
+
+	g_bus_unwatch_name (self->obexd_watch_id);
+	self->obexd_watch_id = 0;
 
 	g_clear_object (&client);
 
@@ -825,55 +792,33 @@ obex_agent_class_init (ObexAgentClass *klass)
 static ObexAgent *
 obex_agent_new (void)
 {
-	ObexAgent *self = NULL;
-
-	self = (ObexAgent *) g_object_new (OBEX_AGENT_TYPE, NULL);
-
-	self->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
-					 AGENT_IFACE,
-					 G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT,
-					 on_bus_acquired,
-					 on_name_acquired,
-					 on_name_lost,
-					 self,
-					 NULL);
-
-	client = bluetooth_client_new ();
-
-	return self;
+	return (ObexAgent *) g_object_new (OBEX_AGENT_TYPE, NULL);
 }
 
 void
 obex_agent_down (void)
 {
-	g_dbus_proxy_call (manager,
-			   "UnregisterAgent",
-			   g_variant_new ("(o)", AGENT_PATH),
-			   G_DBUS_CALL_FLAGS_NONE,
-			   -1,
-			   NULL,
-			   NULL,
-			   NULL);
+	g_dbus_connection_call (agent->connection,
+				MANAGER_SERVICE,
+				MANAGER_PATH,
+				MANAGER_IFACE,
+				"UnregisterAgent",
+				g_variant_new ("(o)", AGENT_PATH),
+				NULL,
+				G_DBUS_CALL_FLAGS_NONE,
+				-1,
+				NULL,
+				NULL,
+				NULL);
 
-	g_clear_object (&manager);
 	g_clear_object (&agent);
 }
 
 gboolean
 obex_agent_up (void)
 {
-	if (agent == NULL) {
+	if (agent == NULL)
 		agent = obex_agent_new ();
-		g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-					  G_DBUS_PROXY_FLAGS_NONE,
-					  NULL,
-					  MANAGER_SERVICE,
-					  MANAGER_PATH,
-					  MANAGER_IFACE,
-					  NULL,
-					  on_proxy_acquired,
-					  NULL);
-	}
 
 	if (!notify_init("gnome-user-share")) {
 		g_warning("Unable to initialize the notification system");
